import com.ericsson.otp.erlang.*;
import java.io.*;
import java.math.*;
import java.util.Random;

/*
  Class JInterfaceServer
  Author: Jacob Torrey <torreyji@clarkson.edu>
  Date: Sept. 25, 2008
  Desc: Provides a BigNum Erlang interface
*/

public class JInterfaceServer {
    private static final boolean DEBUG = false;

    public static void main(String args[]) throws OtpErlangExit, OtpErlangDecodeException, OtpAuthException, IOException, OtpErlangRangeException, OtpErlangExit {
	OtpNode node = new OtpNode("jinterface");
	OtpMbox mbox = node.createMbox("server");
	String cookie;
	if(args.length != 2) {
	    cookie = "ABC";
	} else {
	    cookie = args[1];
	}
	node.setCookie(cookie);
	OtpEpmd.publishPort(node);
	while(true) {
	    OtpErlangTuple command = (OtpErlangTuple) mbox.receive();
	    if(DEBUG)
		System.err.println(command.toString());
	    OtpErlangPid pid = (OtpErlangPid) command.elementAt(0);
	    mbox.link(pid);
	    String test = command.elementAt(1).toString();
	    if(test.equals("quit")) {
		System.exit(0);
	    } else if(test.equals("addint")) { // {Pid, addint, [1, ..., n]} -> {addint_result, sum}
		BigInteger bi = new BigInteger("0");
		OtpErlangList list = (OtpErlangList) command.elementAt(2);
		for(int i = 0; i < list.arity(); i++) {
		    bi = bi.add(((OtpErlangLong) list.elementAt(i)).bigIntegerValue());
		}
		OtpErlangObject[] msg = new OtpErlangObject[2];
		msg[0] = new OtpErlangAtom("addint_result");
		msg[1] = new OtpErlangLong(bi);
		OtpErlangTuple reply = new OtpErlangTuple(msg);
		if(DEBUG)
		    System.err.println("Sending " + reply.toString() + " to " + pid.toString());
		mbox.send(pid, reply);
	    } else if(test.equals("rand")) { // {Pid, rand, bits} -> {rand_result, randnum}
		OtpErlangLong bits = (OtpErlangLong) command.elementAt(2);
		BigInteger bi = new BigInteger(bits.intValue(), new Random());
		OtpErlangObject[] msg = new OtpErlangObject[2];
		msg[0] = new OtpErlangAtom("rand_result");
		msg[1] = new OtpErlangLong(bi);
		OtpErlangTuple reply = new OtpErlangTuple(msg);
		if(DEBUG)
		    System.err.println("Sending " + reply.toString() + " to " + pid.toString());
		mbox.send(pid, reply);
	    } else if(test.equals("gcd")) { // {Pid, gcd, A, B} -> {gcd_result, gcd(A, B)}
		OtpErlangLong a = (OtpErlangLong) command.elementAt(2);
		OtpErlangLong b = (OtpErlangLong) command.elementAt(3);
		BigInteger bia = a.bigIntegerValue();
		BigInteger bib = b.bigIntegerValue();
		bia = bia.gcd(bib);
		OtpErlangObject[] msg = new OtpErlangObject[2];
		msg[0] = new OtpErlangAtom("gcd_result");
		msg[1] = new OtpErlangLong(bia);
		OtpErlangTuple reply = new OtpErlangTuple(msg);
		if(DEBUG)
		    System.err.println("Sending " + reply.toString() + " to " + pid.toString());
		mbox.send(pid, reply);
	    } else if(test.equals("randprime")) { // {Pid, randprime, A, B} -> {randprime_result, randprime}
		OtpErlangLong a = (OtpErlangLong) command.elementAt(2);
		OtpErlangLong b = (OtpErlangLong) command.elementAt(3);
		BigInteger bia = new BigInteger(a.intValue(), b.intValue(), new Random());
		OtpErlangObject[] msg = new OtpErlangObject[2];
		msg[0] = new OtpErlangAtom("randprime_result");
		msg[1] = new OtpErlangLong(bia);
		OtpErlangTuple reply = new OtpErlangTuple(msg);
		if(DEBUG)
		    System.err.println("Sending " + reply.toString() + " to " + pid.toString());
		mbox.send(pid, reply);
	    } else if(test.equals("pow")) { // {Pid, pow, B, E} -> {pow_result, B ^ E)}
		OtpErlangLong B = (OtpErlangLong) command.elementAt(2);
		OtpErlangLong E = (OtpErlangLong) command.elementAt(3);
		BigInteger biB = B.bigIntegerValue();
		int biE = E.intValue();
		biB = biB.pow(biE);
		OtpErlangObject[] msg = new OtpErlangObject[2];
		msg[0] = new OtpErlangAtom("pow_result");
		msg[1] = new OtpErlangLong(biB);
		OtpErlangTuple reply = new OtpErlangTuple(msg);
		if(DEBUG)
		    System.err.println("Sending " + reply.toString() + " to " + pid.toString());
		mbox.send(pid, reply);
	    } else if(test.equals("mod")) { // {Pid, mod, B, E} -> {mod_result, B % E)}
		OtpErlangLong B = (OtpErlangLong) command.elementAt(2);
		OtpErlangLong E = (OtpErlangLong) command.elementAt(3);
		BigInteger biB = B.bigIntegerValue();
		BigInteger biE = E.bigIntegerValue();
		biB = biB.mod(biE);
		OtpErlangObject[] msg = new OtpErlangObject[2];
		msg[0] = new OtpErlangAtom("mod_result");
		msg[1] = new OtpErlangLong(biB);
		OtpErlangTuple reply = new OtpErlangTuple(msg);
		if(DEBUG)
		    System.err.println("Sending " + reply.toString() + " to " + pid.toString());
		mbox.send(pid, reply);
	    } else if(test.equals("modinverse")) { // {Pid, modinverse, B, E} -> {modinverse_result, B^-1 % E)}
		OtpErlangLong B = (OtpErlangLong) command.elementAt(2);
		OtpErlangLong E = (OtpErlangLong) command.elementAt(3);
		BigInteger biB = B.bigIntegerValue();
		BigInteger biE = E.bigIntegerValue();
		biB = biB.modInverse(biE);
		OtpErlangObject[] msg = new OtpErlangObject[2];
		msg[0] = new OtpErlangAtom("modinverse_result");
		msg[1] = new OtpErlangLong(biB);
		OtpErlangTuple reply = new OtpErlangTuple(msg);
		if(DEBUG)
		    System.err.println("Sending " + reply.toString() + " to " + pid.toString());
		mbox.send(pid, reply);
	    } else if(test.equals("modpow")) { // {Pid, modpow, B, E, M} -> {modpow_result, B^E mod M}
		OtpErlangLong B = (OtpErlangLong) command.elementAt(2);
		OtpErlangLong E = (OtpErlangLong) command.elementAt(3);
		OtpErlangLong M = (OtpErlangLong) command.elementAt(4);
		BigInteger BIB = B.bigIntegerValue();
		BigInteger BIE = E.bigIntegerValue();
		BigInteger BIM = M.bigIntegerValue();
		BIB = BIB.modPow(BIE, BIM);
		OtpErlangObject[] msg = new OtpErlangObject[2];
		msg[0] = new OtpErlangAtom("modpow_result");
		msg[1] = new OtpErlangLong(BIB);
		OtpErlangTuple reply = new OtpErlangTuple(msg);
		if(DEBUG)
		    System.err.println("Sending " + reply.toString() + " to " + pid.toString());
		mbox.send(pid, reply);
	    } else {
		mbox.send(pid, new OtpErlangTuple(new OtpErlangAtom("unknown_command")));
	    }
	}
    }
}
