%% @author Jacob Torrey <torreyji@clarkson.edu>
%% @copyright 2008 Jacob Torrey
%% @doc The builtin functions
-module(fang_builtins).
-export([gen_builtins/1]).

% Include our defines
-include("defines.hrl").

%% @doc Saves a the builtins to a file
%% @spec gen_builtins(list()) -> list()
gen_builtins(Filename) ->
    Builtins = [
		{'+', fun(L) ->
			      case ?BIGNUM of
				  false ->
				      F = fun(A, B) ->
						  case A of
						      A when is_number(A) ->
							  A + B;
						      A ->
							  B
						  end end,
				      lists:foldl(F, 0, L);
				  true ->
				      javabignum:addint(lists:filter(fun(E) -> is_number(E) end, L))
			      end end},		
		{'*', fun(L) ->
			      lists:foldl(fun(A, B) ->
						  case A of
						      A when is_number(A) ->
							  A * B;
						      A ->
							  B
						  end end, 1, L) end},
		{'-', fun([H | T]) ->
			      lists:foldl(fun(A, B) ->
						  case A of 
						      A when is_number(A) ->
							  B - A;
						      A ->
							  B
						  end end, H, T) end},
		{'/', fun([H | T]) ->
			      lists:foldl(fun(A, B) ->
						  case A of 
						      A when is_number(A) ->
							  B / A;
						      A ->
							  B
						  end end, H, T) end},
		{'div', fun([H | T]) -> % Integer division
			      lists:foldl(fun(A, B) ->
						  case A of
						      A when is_number(A) ->
							  B div A;
						      A ->
							  B
						  end end, H, T) end},
		{'quit', fun([]) ->
				 init:stop() end},
		{'%', fun([A, B]) ->
			      A rem B end},
		{'defvar', fun([N, V]) ->
				   fang:save_var({fang, N}, V),
				   V end},
		{'not', fun([C]) ->
				case C of
				    true -> false;
				    false -> true
				end end},
		{'==', fun([A, B]) -> A == B end},
		{'<', fun([A, B]) -> A < B end},
		{'>', fun([A, B]) -> A > B end},
		{'or', fun([A, B]) -> case A of
					true -> true;
					false ->
					    case B of
						true -> true;
						false -> false
					    end
				    end end},
		{'and', fun([A, B]) -> case A of
					 true ->
					     case B of
						 true -> true;
						 false -> false
					     end;
					 false -> false
				     end end},
		{'hd', fun([H | T]) ->
			       io:format("HD: ~p ~p ", [H, T]),
			       case T of
				   [] when is_list(H) ->
				       [R | _] = H,
				       io:format("= ~p~n", [R]),
				       R;
				   _ ->
				       io:format("= ~p~n", [H]),
				       H
			       end end},
		{'tl', fun(All) ->
			       case All of
				   [] ->
				       [];
				   _ ->
				       [[_ | T]] = All,
				       T
				   end end},
		{'append', fun([A, B]) -> 
				   lists:append(A, B) end},
		{'cons', fun([A, B]) ->
				 [A, B] end}
		],
    F = fun({N, E}) ->
		fang_functions:write(N, E, Filename) end,
    lists:map(F, Builtins).
