%% @author Jacob Torrey <torreyji@clarkson.edu>
%% @copyright Jacob Torrey 2008
%% @doc A module to serialize/deserialize anonymous functions
-module(fang_functions).
-export([populate/1, write/3]).

%% @doc Populates an ETS table with deserialized functions
%% @spec populate(list()) -> ok
populate(Filename) ->
    {ok, Funcs} = file:consult(Filename),
    Pop = fun(Elem) ->
		  {Name, BinFunc} = Elem,	 
		  fang:save_var(Name, binary_to_term(BinFunc))
	  end,
    lists:foreach(Pop, Funcs),
    ok.

%% @doc Writes a function to a file for loading later
%% @spec write(list(), function(), list()) -> ok
write(Name, Fun, Filename) ->
    {ok, FP} = file:open(Filename, [append, binary]),
    Tuple = {Name, term_to_binary(Fun)},
    io:fwrite(FP, "~p.~n", [Tuple]),
    file:close(FP).
