%% @author Jacob Torrey
%% @copyright 2008 Jacob Torrey
%% @desc A Jinterface libray to provide quick BinNum support
-module(javabignum).
-export([start/0, stop/0, addint/1, gcd/2, modpow/3, pow/2, modinverse/2, mod/2, rand/1, randprime/2]).

% Start the Java bignum process and the Erlang listener
start() ->
    case whereis(jinterfaceserver) of
	undefined ->	
	    os:cmd("./startjava.sh " ++ erlang:atom_to_list(erlang:get_cookie())),
	    RegName = 'server',
	    HostString = erlang:atom_to_list(node()),
	    [_, Host] = string:tokens(HostString, "@"),
	    SendTo = {RegName, erlang:list_to_atom("jinterface@" ++ Host)},
	    erlang:register(jinterfaceserver, spawn(fun() -> loop(SendTo) end)),
	    receive
		1 -> 1
	    after 1000 ->
		    ok
	    end;
	_ ->
	    ok
    end.

% Handle requests
loop(SendTo) ->
    receive
	{From, addint, Ints} ->
	    erlang:send(SendTo, {self(), addint, Ints}, []),
	    receive
		{addint_result, Sum} ->
		    From ! {addint_result, Sum}
	    end,
	    loop(SendTo);
	{From, rand, Bits} ->
	    erlang:send(SendTo, {self(), rand, Bits}, []),
	    receive
		{rand_result, Rand} ->
		    From ! {rand_result, Rand}
	    end,
	    loop(SendTo);
	{From, gcd, A, B} ->
	    erlang:send(SendTo, {self(), gcd, A, B}, []),
	    receive
		{gcd_result, GCD} ->
		    From ! {gcd_result, GCD}
	    end,
	    loop(SendTo);
	{From, randprime, A, B} ->
	    erlang:send(SendTo, {self(), randprime, A, B}, []),
	    receive
		{randprime_result, Rand} ->
		    From ! {randprime_result, Rand}
	    end,
	    loop(SendTo);
	{From, mod, A, B} ->
	    erlang:send(SendTo, {self(), mod, A, B}, []),
	    receive
		{mod_result, Mod} ->
		    From ! {mod_result, Mod}
	    end,
	    loop(SendTo);
	{From, pow, A, B} ->
	    erlang:send(SendTo, {self(), pow, A, B}, []),
	    receive
		{pow_result, Pow} ->
		    From ! {pow_result, Pow}
	    end,
	    loop(SendTo);
	{From, modinverse, A, B} ->
	    erlang:send(SendTo, {self(), modinverse, A, B}, []),
	    receive
		{modinverse_result, Pow} ->
		    From ! {modinverse_result, Pow}
	    end,
	    loop(SendTo);
	{From, modpow, B, E, M} ->
	    erlang:send(SendTo, {self(), modpow, B, E, M}, []),
	    receive
		{modpow_result, MPR} ->
		    From ! {modpow_result, MPR}
	    end,
	    loop(SendTo);
	stop ->
	    erlang:send(SendTo, {self(), quit}, [])
    end.
	    
% Stop both the Erlang and Java processes
stop() ->
    jinterfaceserver ! stop.

% Computes GCD(A, B)
gcd(A, B) ->
    jinterfaceserver ! {self(), gcd, A, B},
    receive
	{gcd_result, GCD} ->
	    GCD
    end.

% Computes A % B
mod(A, B) ->
    jinterfaceserver ! {self(), mod, A, B},
    receive
	{mod_result, Mod} ->
	    Mod
    end.

% Computes the inverse of A mod B
modinverse(A, B) ->
    jinterfaceserver ! {self(), modinverse, A, B},
    receive
	{modinverse_result, Mod} ->
	    Mod
    end.

% Computes B ^ E
pow(B, E) ->
    jinterfaceserver ! {self(), pow, B, E},
    receive
	{pow_result, Pow} ->
	    Pow
    end.

% Computes B ^ E mod M
modpow(B, E, M) ->
    jinterfaceserver ! {self(), modpow, B, E, M},
    receive
	{modpow_result, MPR} ->
	    MPR
    end.

% Returns the sum of the elements of the list
addint(Ints) ->
    jinterfaceserver ! {self(), addint, Ints},
    receive
	{addint_result, Sum} ->
	    Sum
    end.

% Returns a Bits bit random number
rand(Bits) ->
    jinterfaceserver ! {self(), rand, Bits},
    receive
	{rand_result, Rand} ->
	    Rand
    end.

% Returns a Bits bit prime with confidence Conf
randprime(Bits, Conf) ->
    jinterfaceserver ! {self(), randprime, Bits, Conf},
    receive
	{randprime_result, Rand} ->
	    Rand
    end.

